"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoClientFactory = void 0;
const tslib_1 = require("tslib");
const inject_1 = require("@furystack/inject");
const mongodb_1 = require("mongodb");
const semaphore_async_await_1 = tslib_1.__importDefault(require("semaphore-async-await"));
/**
 * Factory for instantiating MongoDb clients
 */
let MongoClientFactory = class MongoClientFactory {
    constructor() {
        this.connections = new Map();
        this.connectionLock = new semaphore_async_await_1.default(1);
    }
    async dispose() {
        await Promise.all([...this.connections.values()].map((c) => c.close()));
        this.connections.clear();
    }
    async getClientFor(url, options) {
        const existing = this.connections.get(url);
        if (existing) {
            return existing;
        }
        try {
            await this.connectionLock.acquire();
            const existingCreated = this.connections.get(url);
            if (existingCreated) {
                return existingCreated;
            }
            const client = await mongodb_1.connect(url, options);
            this.connections.set(url, client);
            return client;
        }
        finally {
            this.connectionLock.release();
        }
    }
};
MongoClientFactory = tslib_1.__decorate([
    inject_1.Injectable({ lifetime: 'singleton' })
], MongoClientFactory);
exports.MongoClientFactory = MongoClientFactory;
//# sourceMappingURL=mongo-client-factory.js.map